#line 2 "pirlexer.c"

#line 4 "pirlexer.c"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5
#define YY_FLEX_SUBMINOR_VERSION 33
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* %if-c++-only */
/* %endif */

/* %if-c-only */

/* %endif */

/* %if-c-only */

/* %endif */

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
/* %if-c-only */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
/* %endif */

/* %if-tables-serialization */
/* %endif */
/* end standard C headers. */

/* %if-c-or-c++ */
/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types.
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t;
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;
#endif /* ! C99 */

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#endif /* ! FLEXINT_H */

/* %endif */

/* %if-c++-only */
/* %endif */

#ifdef __cplusplus

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else   /* ! __cplusplus */

#if __STDC__

#define YY_USE_CONST

#endif  /* __STDC__ */
#endif  /* ! __cplusplus */

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif

/* %not-for-header */

/* Returned upon end-of-file. */
#define YY_NULL 0
/* %ok-for-header */

/* %not-for-header */

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)
/* %ok-for-header */

/* %if-reentrant */

/* An opaque pointer. */
#ifndef YY_TYPEDEF_YY_SCANNER_T
#define YY_TYPEDEF_YY_SCANNER_T
typedef void* yyscan_t;
#endif

/* For convenience, these vars (plus the bison vars far below)
   are macros in the reentrant scanner. */
#define yyin yyg->yyin_r
#define yyout yyg->yyout_r
#define yyextra yyg->yyextra_r
#define yyleng yyg->yyleng_r
#define yytext yyg->yytext_r
#define yylineno (YY_CURRENT_BUFFER_LVALUE->yy_bs_lineno)
#define yycolumn (YY_CURRENT_BUFFER_LVALUE->yy_bs_column)
#define yy_flex_debug yyg->yy_flex_debug_r

int yylex_init (yyscan_t* scanner);
/* %endif */

/* %if-not-reentrant */
/* %endif */

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN yyg->yy_start = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START ((yyg->yy_start - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE yyrestart(yyin ,yyscanner )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#define YY_BUF_SIZE 16384
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

/* %if-not-reentrant */
/* %endif */

/* %if-c-only */
/* %if-not-reentrant */
/* %endif */
/* %endif */

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

    #define YY_LESS_LINENO(n)

/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
    do \
        { \
        /* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
        *yy_cp = yyg->yy_hold_char; \
        YY_RESTORE_YY_MORE_OFFSET \
        yyg->yy_c_buf_p = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
        YY_DO_BEFORE_ACTION; /* set up yytext again */ \
        } \
    while ( 0 )

#define unput(c) yyunput( c, yyg->yytext_ptr , yyscanner )

/* The following is because we cannot portably get our hands on size_t
 * (without autoconf's help, which isn't available because we want
 * flex-generated scanners to compile on their own).
 */

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef unsigned int yy_size_t;
#endif

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
    {
/* %if-c-only */
    FILE *yy_input_file;
/* %endif */

/* %if-c++-only */
/* %endif */

    char *yy_ch_buf;        /* input buffer */
    char *yy_buf_pos;       /* current position in input buffer */

    /* Size of input buffer in bytes, not including room for EOB
     * characters.
     */
    yy_size_t yy_buf_size;

    /* Number of characters read into yy_ch_buf, not including EOB
     * characters.
     */
    int yy_n_chars;

    /* Whether we "own" the buffer - i.e., we know we created it,
     * and can realloc() it to grow it, and should free() it to
     * delete it.
     */
    int yy_is_our_buffer;

    /* Whether this is an "interactive" input source; if so, and
     * if we're using stdio for input, then we want to use getc()
     * instead of fread(), to make sure we stop fetching input after
     * each newline.
     */
    int yy_is_interactive;

    /* Whether we're considered to be at the beginning of a line.
     * If so, '^' rules will be active on the next match, otherwise
     * not.
     */
    int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */

    /* Whether to try to fill the input buffer when we reach the
     * end of it.
     */
    int yy_fill_buffer;

    int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
    /* When an EOF's been seen but there's still some text to process
     * then we mark the buffer as YY_EOF_PENDING, to indicate that we
     * shouldn't try reading from the input source any more.  We might
     * still have a bunch of tokens to match, though, because of
     * possible backing-up.
     *
     * When we actually see the EOF, we change the status to "new"
     * (via yyrestart()), so that the user can continue scanning by
     * just pointing yyin at a new input file.
     */
#define YY_BUFFER_EOF_PENDING 2

    };
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* %if-c-only Standard (non-C++) definition */
/* %not-for-header */

/* %if-not-reentrant */
/* %endif */
/* %ok-for-header */

/* %endif */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( yyg->yy_buffer_stack \
                          ? yyg->yy_buffer_stack[yyg->yy_buffer_stack_top] \
                          : NULL)

/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE yyg->yy_buffer_stack[yyg->yy_buffer_stack_top]

/* %if-c-only Standard (non-C++) definition */

/* %if-not-reentrant */
/* %not-for-header */

/* %ok-for-header */

/* %endif */

void yyrestart (FILE *input_file ,yyscan_t yyscanner );
void yy_switch_to_buffer (YY_BUFFER_STATE new_buffer ,yyscan_t yyscanner );
YY_BUFFER_STATE yy_create_buffer (FILE *file,int size ,yyscan_t yyscanner );
void yy_delete_buffer (YY_BUFFER_STATE b ,yyscan_t yyscanner );
void yy_flush_buffer (YY_BUFFER_STATE b ,yyscan_t yyscanner );
void yypush_buffer_state (YY_BUFFER_STATE new_buffer ,yyscan_t yyscanner );
void yypop_buffer_state (yyscan_t yyscanner );

static void yyensure_buffer_stack (yyscan_t yyscanner );
static void yy_load_buffer_state (yyscan_t yyscanner );
static void yy_init_buffer (YY_BUFFER_STATE b,FILE *file ,yyscan_t yyscanner );

#define YY_FLUSH_BUFFER yy_flush_buffer(YY_CURRENT_BUFFER ,yyscanner)

YY_BUFFER_STATE yy_scan_buffer (char *base,yy_size_t size ,yyscan_t yyscanner );
YY_BUFFER_STATE yy_scan_string (yyconst char *yy_str ,yyscan_t yyscanner );
YY_BUFFER_STATE yy_scan_bytes (yyconst char *bytes,int len ,yyscan_t yyscanner );

/* %endif */

void *yyalloc (yy_size_t ,yyscan_t yyscanner );
void *yyrealloc (void *,yy_size_t ,yyscan_t yyscanner );
void yyfree (void * ,yyscan_t yyscanner );

#define yy_new_buffer yy_create_buffer

#define yy_set_interactive(is_interactive) \
    { \
    if ( ! YY_CURRENT_BUFFER ){ \
        yyensure_buffer_stack (yyscanner); \
        YY_CURRENT_BUFFER_LVALUE =    \
            yy_create_buffer(yyin,YY_BUF_SIZE ,yyscanner); \
    } \
    YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
    }

#define yy_set_bol(at_bol) \
    { \
    if ( ! YY_CURRENT_BUFFER ){\
        yyensure_buffer_stack (yyscanner); \
        YY_CURRENT_BUFFER_LVALUE =    \
            yy_create_buffer(yyin,YY_BUF_SIZE ,yyscanner); \
    } \
    YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
    }

#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* %% [1.0] yytext/yyin/yyout/yy_state_type/yylineno etc. def's & init go here */
/* Begin user sect3 */

#define yywrap(n) 1
#define YY_SKIP_YYWRAP

#define FLEX_DEBUG

typedef unsigned char YY_CHAR;

typedef int yy_state_type;

#define yytext_ptr yytext_r

/* %if-c-only Standard (non-C++) definition */

static yy_state_type yy_get_previous_state (yyscan_t yyscanner );
static yy_state_type yy_try_NUL_trans (yy_state_type current_state  ,yyscan_t yyscanner);
static int yy_get_next_buffer (yyscan_t yyscanner );
static void yy_fatal_error (yyconst char msg[] ,yyscan_t yyscanner );

/* %endif */

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
    yyg->yytext_ptr = yy_bp; \
/* %% [2.0] code to fiddle yytext and yyleng for yymore() goes here \ */\
    yyleng = (size_t) (yy_cp - yy_bp); \
    yyg->yy_hold_char = *yy_cp; \
    *yy_cp = '\0'; \
/* %% [3.0] code to copy yytext_ptr to yytext[] goes here, if %array \ */\
    yyg->yy_c_buf_p = yy_cp;

/* %% [4.0] data tables for the DFA and the user's section 1 definitions go here */
#define YY_NUM_RULES 117
#define YY_END_OF_BUFFER 118
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info
    {
    flex_int32_t yy_verify;
    flex_int32_t yy_nxt;
    };
static yyconst flex_int16_t yy_accept[391] =
    {   0,
        0,    0,  118,  116,    1,    2,    1,   23,  116,  116,
       20,  116,  116,   26,   27,   21,   19,   28,   25,   32,
       22,  112,  112,  116,   34,   13,   33,   14,  110,  110,
      110,  110,  110,   29,   30,  110,  110,  110,  110,  110,
      110,  116,   24,    0,    2,    2,   10,    0,  100,    0,
        0,    0,    0,    0,   39,   16,   42,    0,    0,   38,
        0,  112,   35,   36,  111,   45,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
       15,   37,  111,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    7,   11,

        9,    8,   12,    6,  110,  109,  104,  103,  101,  102,
      110,   46,  110,  110,  110,  110,  110,  110,   41,   17,
       44,   18,   31,  108,  107,  105,  106,   40,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,   43,  111,
      114,    0,  111,  115,  113,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    5,    4,  110,   51,  110,  110,   52,   53,  110,
      110,    0,  111,   60,   55,    0,    0,    0,   57,    0,
        0,    0,   63,    0,    0,    0,    0,    0,    0,    0,

        0,    0,   80,    0,    0,    0,    0,    0,    0,   89,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    3,   47,  110,   50,  110,  110,    0,    0,   73,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,   82,   98,    0,   83,    0,
       84,   87,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,  110,  110,  110,    0,    0,   56,    0,    0,
        0,    0,    0,    0,    0,   65,    0,    0,    0,   69,
        0,    0,    0,   81,    0,    0,    0,   92,   97,    0,
        0,   90,    0,    0,    0,    0,  110,   54,   49,    0,

        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,   77,   78,   79,    0,    0,   88,    0,    0,    0,
       96,    0,   91,  110,   61,    0,    0,    0,    0,    0,
        0,    0,    0,    0,   64,    0,    0,    0,    0,    0,
        0,    0,    0,    0,  110,    0,    0,    0,   74,    0,
        0,    0,    0,   62,    0,    0,   68,    0,   99,   95,
       94,   85,    0,  110,    0,    0,    0,    0,   76,    0,
        0,   66,   67,   86,    0,  110,   70,    0,    0,   75,
        0,    0,   93,   48,    0,   72,   58,   59,   71,    0
    } ;

static yyconst flex_int32_t yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    4,    5,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    4,    1,    1,    1,    1,
        1,    2,    6,    7,    1,    8,    9,   10,   11,   12,
       13,   14,   15,   16,   17,   18,   19,   20,   21,   22,
       22,   22,   22,   22,   22,   23,   23,   24,   25,   26,
       27,   28,    1,   29,   30,   31,   30,   30,   32,   30,
       29,   33,   34,   29,   29,   35,   29,   36,   37,   38,
       29,   29,   39,   29,   29,   29,   29,   40,   29,   29,
       41,   42,   43,    1,   44,    1,   45,   46,   47,   48,

       49,   50,   51,   52,   53,   29,   29,   54,   55,   56,
       57,   58,   59,   60,   61,   62,   63,   64,   29,   65,
       66,   29,    1,   67,    1,   68,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst flex_int32_t yy_meta[69] =
    {   0,
        1,    1,    2,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    3,
        3,    3,    3,    4,    1,    1,    1,    1,    4,    3,
        3,    3,    4,    4,    4,    4,    4,    4,    4,    4,
        1,    1,    1,    4,    3,    3,    3,    3,    3,    3,
        4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
        4,    4,    4,    4,    4,    4,    1,    1
    } ;

static yyconst flex_int16_t yy_base[395] =
    {   0,
        0,    0,  623,  624,  604,   67,   68,  594,   66,   40,
      593,   65,  608,  624,  624,   63,   62,  624,   73,   89,
       78,  138,   81,  159,  624,   61,   79,   87,  594,   97,
      103,  142,  159,  624,  624,   74,  117,  108,  113,  115,
      129,  119,  106,  186,  191,  195,  624,  147,  624,    0,
      185,  204,  208,  212,  624,  624,  624,  606,  589,  624,
      216,  222,  624,  624,  226,  624,  580,  554,  564,  154,
      556,  201,  555,  153,  561,  174,  112,  560,  545,  554,
      579,  624,  240,  108,  259,  231,    0,  549,  550,  201,
      210,  220,  558,  210,  545,  547,  544,  537,  624,  624,

      624,  624,  624,  250,  574,  624,  264,  270,  275,  280,
      246,  573,  281,  282,  283,  194,  285,  286,  624,  624,
      624,  624,  624,  291,  295,  299,  303,  624,  312,  561,
      544,  543,  539,  536,  543,  528,  532,  524,  522,  219,
      524,  530,  531,  523,  537,  269,  535,  531,  624,  326,
      321,  330,  334,  339,    0,  522,  533,  522,  309,  511,
      530,  521,  511,  518,  516,  508,  507,  507,  504,  513,
      520,  624,  537,  312,  539,  320,  339,  538,  537,  341,
      127,  359,  363,  516,  624,  506,  504,  496,  512,  511,
      508,  496,  624,  504,  506,  498,  500,  504,  502,  495,

      482,  481,  624,  489,  486,  479,  491,  477,  481,  624,
      489,  480,  483,  472,  484,  148,  483,  469,  470,  470,
      482,  624,  503,  340,  502,  342,  343,  470,  468,  624,
      461,  330,  462,  476,  473,  465,  464,  473,  455,  468,
      459,  458,  458,  451,  462,  624,  624,  461,  624,  461,
      624,  624,  450,  453,  457,  454,  446,  442,  454,  442,
      436,  444,  346,  344,  347,  452,  452,  624,  450,  445,
      440,  443,  437,  445,  436,  624,  441,  429,  441,  624,
      440,  422,  427,  624,  429,  436,  432,  624,  624,  425,
      422,  624,  420,  410,  426,  425,  352,  449,  448,  413,

      341,  416,  407,  419,  406,  419,  409,  418,  418,  417,
      407,  624,  624,  624,  415,  403,  624,  413,  412,  401,
      624,  411,  624,  348,  624,  409,  404,  399,  397,  387,
      395,  385,  390,  384,  624,  391,  397,  389,  380,  379,
      389,  385,  380,  373,  363,  375,  364,  376,  624,  364,
      375,  368,  365,  624,  365,  369,  624,  368,  624,  624,
      624,  624,  367,  367,  361,  351,  359,  356,  624,  347,
      344,  624,  624,  624,  352,  350,  624,  342,  352,  624,
      338,  330,  624,  304,   92,  624,  624,  624,  624,  624,
      427,  431,  433,   78

    } ;

static yyconst flex_int16_t yy_def[395] =
    {   0,
      390,    1,  390,  390,  390,  390,  390,  390,  391,  390,
      390,  390,  392,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  393,  393,
      393,  393,  393,  390,  390,  393,  393,  393,  393,  393,
      393,  390,  390,  390,  390,  390,  390,  391,  390,  391,
      390,  390,  390,  390,  390,  390,  390,  392,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  394,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,

      390,  390,  390,  390,  393,  390,  393,  393,  393,  393,
      393,  393,  393,  393,  393,  393,  393,  393,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  394,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  393,  393,  393,  393,  393,  393,  393,
      393,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,

      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  393,  393,  393,  393,  393,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  393,  393,  393,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  393,  393,  393,  390,

      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  393,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  393,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  393,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  393,  390,  390,  390,  390,
      390,  390,  390,  393,  390,  390,  390,  390,  390,    0,
      390,  390,  390,  390

    } ;

static yyconst flex_int16_t yy_nxt[693] =
    {   0,
        4,    5,    6,    5,    7,    8,    9,   10,   11,   12,
       13,   14,   15,   16,   17,   18,   19,   20,   21,   22,
       23,   23,   23,   24,   25,   26,   27,   28,   29,   29,
       29,   29,   29,   30,   29,   31,   29,   32,   33,   29,
       34,    4,   35,   29,   29,   29,   29,   29,   29,   29,
       36,   29,   37,   29,   29,   38,   29,   39,   29,   29,
       40,   29,   41,   29,   29,   29,   42,   43,   45,   45,
       46,   45,   49,   51,   56,   52,   59,   53,   54,   61,
      155,   62,   62,   62,   62,   44,   99,  100,   63,   60,
       61,   57,   62,   62,   62,   62,   81,  106,   83,   64,

       62,   62,   62,   62,   82,  101,  102,   50,   65,   65,
       65,   65,   85,  103,  104,   66,  107,  107,  107,  107,
      106,   67,  108,  108,  108,  108,  106,  151,  151,   85,
      111,  106,  121,   68,   69,   70,  106,   71,  106,   72,
      106,   73,   74,   75,   76,  119,   77,  389,   78,   79,
      106,  114,  106,   49,   80,   83,  144,   62,   62,   62,
       62,  109,  109,  109,  109,  106,  112,  116,   84,   85,
      115,  145,  113,  122,   86,  227,  117,   87,  110,  110,
      110,  110,  106,   84,  118,  120,   85,  123,   50,  123,
      123,  256,   45,   45,   86,   45,   45,   45,  133,   45,

      257,  139,   87,   88,  124,  124,  124,  124,   89,  140,
      134,   90,   91,   92,   93,   94,   95,  106,  142,   96,
      143,   97,   98,  125,  125,  125,  125,  126,  126,  126,
      126,  127,  127,  127,  127,   65,   65,   65,   65,   83,
      179,   62,   62,   62,   62,   65,   65,   65,   65,  136,
      154,  154,  154,   85,  137,  158,  159,  129,  160,  150,
      150,  150,  150,  194,  162,  195,  161,  166,  163,  106,
       85,  129,  167,  152,  129,  152,  172,  173,  153,  153,
      153,  153,  164,  107,  107,  107,  107,  106,  129,  108,
      108,  108,  108,  106,  109,  109,  109,  109,  106,  110,

      110,  110,  110,  106,  106,  106,  106,  174,  106,  106,
      124,  124,  124,  124,  125,  125,  125,  125,  126,  126,
      126,  126,  127,  127,  127,  127,  182,  106,  182,  201,
      202,  183,  183,  183,  183,  106,  177,  178,  176,  181,
      151,  151,  175,  106,  180,  150,  150,  150,  150,  153,
      153,  153,  153,  153,  153,  153,  153,  129,  154,  154,
      154,  208,  106,  106,  106,  106,  106,  106,  223,  106,
      106,  106,  209,  106,  129,  106,  269,  224,  183,  183,
      183,  183,  183,  183,  183,  183,  106,  326,  263,  270,
      106,  388,  225,  226,  298,  271,  324,  264,  387,  386,

      327,  385,  383,  265,  382,  297,  328,  299,  381,  345,
      384,  380,  379,  378,  377,  375,  374,  373,  372,  364,
      371,  370,  369,  368,  367,  366,  376,   48,  365,   48,
       48,   58,  363,   58,   58,  105,  105,  362,  361,  360,
      359,  358,  357,  356,  355,  354,  353,  352,  351,  350,
      349,  348,  347,  346,  344,  343,  342,  341,  340,  339,
      338,  337,  336,  335,  334,  333,  332,  331,  330,  329,
      325,  106,  106,  323,  322,  321,  320,  319,  318,  317,
      316,  315,  314,  313,  312,  311,  310,  309,  308,  307,
      306,  305,  304,  303,  302,  301,  300,  296,  295,  294,

      293,  292,  291,  290,  289,  288,  287,  286,  285,  284,
      283,  282,  281,  280,  279,  278,  277,  276,  275,  274,
      273,  272,  268,  267,  266,  106,  106,  262,  261,  260,
      259,  258,  255,  254,  253,  252,  251,  250,  249,  248,
      247,  246,  245,  244,  243,  242,  241,  240,  239,  238,
      237,  236,  235,  234,  233,  232,  231,  230,  229,  228,
      106,  106,  106,  222,  221,  220,  219,  218,  217,  216,
      215,  214,  213,  212,  211,  210,  207,  206,  205,  204,
      203,  200,  199,  198,  197,  196,  193,  192,  191,  190,
      189,  188,  187,  186,  185,  184,  106,  106,  171,  170,

      169,  168,  165,  157,  156,  149,  148,  147,  146,  141,
      138,  135,  132,  131,  130,  128,   49,  106,   49,   55,
       47,   44,  390,    3,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390

    } ;

static yyconst flex_int16_t yy_chk[693] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    6,    6,
        7,    6,    9,   10,   12,   10,   16,   10,   10,   17,
      394,   17,   17,   17,   17,    7,   26,   26,   17,   16,
       19,   12,   19,   19,   19,   19,   21,   36,   23,   19,

       23,   23,   23,   23,   21,   27,   27,    9,   20,   20,
       20,   20,   23,   28,   28,   20,   30,   30,   30,   30,
       30,   20,   31,   31,   31,   31,   31,   84,   84,   23,
       36,   38,   43,   20,   20,   20,   39,   20,   40,   20,
       37,   20,   20,   20,   20,   42,   20,  385,   20,   20,
      181,   38,   41,   48,   20,   22,   77,   22,   22,   22,
       22,   32,   32,   32,   32,   32,   37,   39,   22,   22,
       38,   77,   37,   43,   22,  181,   40,   22,   33,   33,
       33,   33,   33,   22,   41,   42,   22,   44,   48,   44,
       44,  216,   45,   45,   22,   45,   46,   46,   70,   46,

      216,   74,   22,   24,   51,   51,   51,   51,   24,   74,
       70,   24,   24,   24,   24,   24,   24,  116,   76,   24,
       76,   24,   24,   52,   52,   52,   52,   53,   53,   53,
       53,   54,   54,   54,   54,   61,   61,   61,   61,   62,
      116,   62,   62,   62,   62,   65,   65,   65,   65,   72,
       86,   86,   86,   62,   72,   90,   90,   65,   91,   83,
       83,   83,   83,  140,   92,  140,   91,   94,   92,  111,
       62,   83,   94,   85,   65,   85,  104,  104,   85,   85,
       85,   85,   92,  107,  107,  107,  107,  107,   83,  108,
      108,  108,  108,  108,  109,  109,  109,  109,  109,  110,

      110,  110,  110,  110,  113,  114,  115,  111,  117,  118,
      124,  124,  124,  124,  125,  125,  125,  125,  126,  126,
      126,  126,  127,  127,  127,  127,  129,  384,  129,  146,
      146,  129,  129,  129,  129,  174,  115,  115,  114,  118,
      151,  151,  113,  176,  117,  150,  150,  150,  150,  152,
      152,  152,  152,  153,  153,  153,  153,  150,  154,  154,
      154,  159,  177,  224,  180,  226,  227,  264,  174,  263,
      265,  324,  159,  376,  150,  297,  232,  176,  182,  182,
      182,  182,  183,  183,  183,  183,  345,  301,  224,  232,
      364,  382,  177,  180,  264,  232,  297,  226,  381,  379,

      301,  378,  375,  227,  371,  263,  301,  265,  370,  324,
      376,  368,  367,  366,  365,  363,  358,  356,  355,  345,
      353,  352,  351,  350,  348,  347,  364,  391,  346,  391,
      391,  392,  344,  392,  392,  393,  393,  343,  342,  341,
      340,  339,  338,  337,  336,  334,  333,  332,  331,  330,
      329,  328,  327,  326,  322,  320,  319,  318,  316,  315,
      311,  310,  309,  308,  307,  306,  305,  304,  303,  302,
      300,  299,  298,  296,  295,  294,  293,  291,  290,  287,
      286,  285,  283,  282,  281,  279,  278,  277,  275,  274,
      273,  272,  271,  270,  269,  267,  266,  262,  261,  260,

      259,  258,  257,  256,  255,  254,  253,  250,  248,  245,
      244,  243,  242,  241,  240,  239,  238,  237,  236,  235,
      234,  233,  231,  229,  228,  225,  223,  221,  220,  219,
      218,  217,  215,  214,  213,  212,  211,  209,  208,  207,
      206,  205,  204,  202,  201,  200,  199,  198,  197,  196,
      195,  194,  192,  191,  190,  189,  188,  187,  186,  184,
      179,  178,  175,  173,  171,  170,  169,  168,  167,  166,
      165,  164,  163,  162,  161,  160,  158,  157,  156,  148,
      147,  145,  144,  143,  142,  141,  139,  138,  137,  136,
      135,  134,  133,  132,  131,  130,  112,  105,   98,   97,

       96,   95,   93,   89,   88,   81,   80,   79,   78,   75,
       73,   71,   69,   68,   67,   59,   58,   29,   13,   11,
        8,    5,    3,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390,  390,  390,  390,  390,  390,  390,  390,  390,
      390,  390

    } ;

static yyconst flex_int16_t yy_rule_linenum[117] =
    {   0,
      271,  273,  277,  278,  279,  280,  281,  282,  283,  284,
      285,  286,  287,  288,  290,  291,  292,  293,  295,  296,
      297,  298,  299,  300,  301,  302,  303,  304,  305,  306,
      308,  311,  312,  313,  315,  316,  317,  318,  319,  320,
      321,  322,  323,  324,  325,  327,  328,  329,  330,  331,
      333,  334,  335,  336,  338,  339,  340,  342,  343,  344,
      345,  346,  347,  348,  349,  351,  352,  353,  354,  355,
      356,  357,  358,  359,  360,  361,  362,  363,  364,  365,
      366,  368,  369,  370,  371,  372,  373,  374,  375,  376,
      377,  378,  380,  381,  382,  383,  384,  385,  386,  389,

      394,  395,  396,  397,  399,  400,  401,  402,  404,  409,
      415,  416,  417,  418,  419,  424
    } ;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
#line 1 "pir.l"
#line 2 "pir.l"

/*
 * $Id: pirlexer.c 24306 2007-12-30 15:32:04Z kjs $
 * Copyright (C) 2007, The Perl Foundation.
 */

/*

=head1 NAME

pir.l

=head1 DESCRIPTION

This is a complete rewrite of the PIR lexical analyzer, as defined in IMCC.
Goal is to fix the issues with the current implementation of the PIR language.

The current approach is to create a three-pass compiler, but if any optimizations
in this schedule can be made, then this is preferred. This needs more experimentation.

The first pass is the heredoc pre-processor, which converts all heredoc strings into
normal strings (they are "flattened). Furthermore, this phase strips all comments, both
POD and line comments.

The second pass is the macro pre-processor, which handles the C<.macro>, C<.macro_const>
and C<.include> directives. The resulting output is the file that can be fed into the
actual PIR parser.

The third pass is then the PIR parsing phase. It takes the output of the macro pre-processor,
which contains no heredoc strings and macros. For that reason, the PIR lexer is very
simple and straightforward.

Each of the phases can be easily implemented. When they must be combined, the complexity
grows quickly. Therefore, this approach, which is probably not the most efficient, is
easier to maintain, and preferable.


=cut

*/

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>
#include "pirparser.h"
#include "pircompiler.h"

/* Windows doesn't have <unistd.h> */
#define YY_NO_UNISTD_H

/* define the type of the extra field in the yyscan_t object that is passed around */
#define YY_EXTRA_TYPE  struct lexer_state *

/* accessor methods for setting and getting the lexer_state */
extern YY_EXTRA_TYPE  yyget_extra(yyscan_t scanner);
extern void yyset_extra(YY_EXTRA_TYPE lexer ,yyscan_t scanner);

/* accessor method to get yytext */
extern char *yyget_text(yyscan_t yyscanner);

/* declaration of yylex */
extern int yylex(YYSTYPE *yylval,yyscan_t yyscanner);

extern void syntax_error(yyscan_t yyscanner, lexer_state *lexer, char *message);



/* keep MSVC happy */
#ifndef YY_MAIN
#  define YY_MAIN 0
#endif

/* keep MSVC happy */
#ifndef YY_ALWAYS_INTERACTIVE
#  define YY_ALWAYS_INTERACTIVE 0
#endif



/* Parrot can check out whether the specified text is the name of an op.
 * We define a dummy function for now; replace this later.
 */
static int is_parrot_op(char const * const spelling);



#define DEBUG1
/* think of a smarter way to do this; only print when DEBUG is defined */
#ifdef DEBUG
#  define printdebug fprintf
#else
#  define printdebug noprint
void noprint(FILE *fp, char *format, ...) { }
#endif



/*

=over 4

=item C<update_location>

Update the line number. The yytext buffer is scanned for '\n'
characters; for each one, the line number is incremented. It's done this
way, because we can't increment it in the rule for matching newlines, as
a set of consecutive newlines are matched together and a single newline
token is returned.

*/
static void
update_location(void *yyscanner, lexer_state * const lexer) {
    char const *iter = yyget_text(yyscanner);
    assert(lexer != NULL);

    /* TODO: is yytext NULL terminated? */
    while (*iter != '\0') {
        if (*iter == '\n') {
            ++lexer->line_nr;
            lexer->line_pos = 1; /* reset column */
        }
        else {
            ++lexer->line_pos;
        }
        iter++;
    }
}

/*

=item C<dupstr>

The C89 standard does not define a strdup() in the C library,
so define our own strdup. Function names beginning with "str"
are reserved (I think), so make it dupstr, as that is what it
does: duplicate a string.

*/
static char *
dupstr(char const * const source) {
    char *newstring = (char *)calloc(strlen(source) + 1, sizeof (char));
    assert(newstring);
    strcpy(newstring, source);
    return newstring;
}

/*

=item C<dupstrn>

See dupstr, except that this version takes the number of characters to be
copied. Easy for copying a string except the quotes.

*/
static char *
dupstrn(char const * const source, size_t num_chars) {
    char *newstring = (char *)calloc(num_chars + 1, sizeof (char));
    assert(newstring);
    /* only copy num_chars characters */
    strncpy(newstring, source, num_chars);
    return newstring;
}


/* after each rule execute update_location() */
#define YY_USER_ACTION  do {                                                \
                            lexer_state *my_lexer = yyget_extra(yyscanner); \
                            update_location(yyscanner, my_lexer);           \
                        }                                                   \
                        while(0);





/*

=item C<new_lexer>

constructor for a lexer. It's very important to initialize all fields.

*/
lexer_state *
new_lexer(char * const filename) {
    lexer_state *lexer   = (lexer_state *)malloc(sizeof (lexer_state));
    assert(lexer != NULL);

    lexer->filename      = filename;
    lexer->line_nr       = 1;
    lexer->line_pos      = 1;
    lexer->parse_errors  = 0;

    lexer->subs          = NULL;
    lexer->is_instr      = 0;

    printdebug(stderr, "Constructing new lexer\n");

    return lexer;
}

/*

=back

=cut

*/




/* make sure yytext is a pointer */
/* slightly more efficient when this option is set; our parser is not interactive anyway. */
/* define output file */
/* make the scanner re-entrant */
/* needed for bison interaction. I forgot details. */
/* make yywrap() always return true. */
/* always show warnings if something's wrong with our spec. */
/* create a scanner in debug mode */
#line 1037 "pirlexer.c"

#define INITIAL 0

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
/* %if-c-only */
#include <unistd.h>
/* %endif */
/* %if-c++-only */
/* %endif */
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

/* %if-c-only Reentrant structure and macros (non-C++). */
/* %if-reentrant */

/* Holds the entire state of the reentrant scanner. */
struct yyguts_t
    {

    /* User-defined. Not touched by flex. */
    YY_EXTRA_TYPE yyextra_r;

    /* The rest are the same as the globals declared in the non-reentrant scanner. */
    FILE *yyin_r, *yyout_r;
    size_t yy_buffer_stack_top; /**< index of top of stack. */
    size_t yy_buffer_stack_max; /**< capacity of stack. */
    YY_BUFFER_STATE * yy_buffer_stack; /**< Stack as an array. */
    char yy_hold_char;
    int yy_n_chars;
    int yyleng_r;
    char *yy_c_buf_p;
    int yy_init;
    int yy_start;
    int yy_did_buffer_switch_on_eof;
    int yy_start_stack_ptr;
    int yy_start_stack_depth;
    int *yy_start_stack;
    yy_state_type yy_last_accepting_state;
    char* yy_last_accepting_cpos;

    int yylineno_r;
    int yy_flex_debug_r;

    char *yytext_r;
    int yy_more_flag;
    int yy_more_len;

    YYSTYPE * yylval_r;

    }; /* end struct yyguts_t */

/* %if-c-only */

static int yy_init_globals (yyscan_t yyscanner );

/* %endif */

/* %if-reentrant */

    /* This must go here because YYSTYPE and YYLTYPE are included
     * from bison output in section 1.*/
    #    define yylval yyg->yylval_r

/* %endif */

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int yylex_destroy (yyscan_t yyscanner );

int yyget_debug (yyscan_t yyscanner );

void yyset_debug (int debug_flag ,yyscan_t yyscanner );

YY_EXTRA_TYPE yyget_extra (yyscan_t yyscanner );

void yyset_extra (YY_EXTRA_TYPE user_defined ,yyscan_t yyscanner );

FILE *yyget_in (yyscan_t yyscanner );

void yyset_in  (FILE * in_str ,yyscan_t yyscanner );

FILE *yyget_out (yyscan_t yyscanner );

void yyset_out  (FILE * out_str ,yyscan_t yyscanner );

int yyget_leng (yyscan_t yyscanner );

char *yyget_text (yyscan_t yyscanner );

int yyget_lineno (yyscan_t yyscanner );

void yyset_lineno (int line_number ,yyscan_t yyscanner );

/* %if-bison-bridge */

YYSTYPE * yyget_lval (yyscan_t yyscanner );

void yyset_lval (YYSTYPE * yylval_param ,yyscan_t yyscanner );

/* %endif */
/* %endif End reentrant structures and macros. */

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int yywrap (yyscan_t yyscanner );
#else
extern int yywrap (yyscan_t yyscanner );
#endif
#endif

/* %not-for-header */

/* %ok-for-header */

/* %endif */

#ifndef yytext_ptr
static void yy_flex_strncpy (char *,yyconst char *,int ,yyscan_t yyscanner);
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * ,yyscan_t yyscanner);
#endif

#ifndef YY_NO_INPUT
/* %if-c-only Standard (non-C++) definition */
/* %not-for-header */

#ifdef __cplusplus
static int yyinput (yyscan_t yyscanner );
#else
static int input (yyscan_t yyscanner );
#endif
/* %ok-for-header */

/* %endif */
#endif

/* %if-c-only */

/* %endif */

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#define YY_READ_BUF_SIZE 8192
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* %if-c-only Standard (non-C++) definition */
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO (void) fwrite( yytext, yyleng, 1, yyout )
/* %endif */
/* %if-c++-only C++ definition */
/* %endif */
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
/* %% [5.0] fread()/read() definition of YY_INPUT goes here unless we're doing C++ \ */\
    if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
        { \
        int c = '*'; \
        size_t n; \
        for ( n = 0; n < max_size && \
                 (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
            buf[n] = (char) c; \
        if ( c == '\n' ) \
            buf[n++] = (char) c; \
        if ( c == EOF && ferror( yyin ) ) \
            YY_FATAL_ERROR( "input in flex scanner failed" ); \
        result = n; \
        } \
    else \
        { \
        errno=0; \
        while ( (result = fread(buf, 1, max_size, yyin))==0 && ferror(yyin)) \
            { \
            if( errno != EINTR) \
                { \
                YY_FATAL_ERROR( "input in flex scanner failed" ); \
                break; \
                } \
            errno=0; \
            clearerr(yyin); \
            } \
        }\
\
/* %if-c++-only C++ definition \ */\
/* %endif */

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
/* %if-c-only */
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg , yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
#endif

/* %if-tables-serialization structures and prototypes */
/* %not-for-header */

/* %ok-for-header */

/* %not-for-header */

/* %tables-yydmap generated elements */
/* %endif */
/* end tables serialization structures and prototypes */

/* %ok-for-header */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1
/* %if-c-only Standard (non-C++) definition */

extern int yylex (YYSTYPE * yylval_param ,yyscan_t yyscanner);

#define YY_DECL int yylex (YYSTYPE * yylval_param , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only C++ definition */
/* %endif */
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

/* %% [6.0] YY_RULE_SETUP definition goes here */
#define YY_RULE_SETUP \
    YY_USER_ACTION

/* %not-for-header */

/** The main scanner function which does all the work.
 */
YY_DECL
{
    register yy_state_type yy_current_state;
    register char *yy_cp, *yy_bp;
    register int yy_act;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

/* %% [7.0] user's declarations go here */
#line 268 "pir.l"



#line 1330 "pirlexer.c"

    yylval = yylval_param;

    if ( !yyg->yy_init )
        {
        yyg->yy_init = 1;

#ifdef YY_USER_INIT
        YY_USER_INIT;
#endif

        if ( ! yyg->yy_start )
            yyg->yy_start = 1;  /* first start state */

        if ( ! yyin )
/* %if-c-only */
            yyin = stdin;
/* %endif */
/* %if-c++-only */
/* %endif */

        if ( ! yyout )
/* %if-c-only */
            yyout = stdout;
/* %endif */
/* %if-c++-only */
/* %endif */

        if ( ! YY_CURRENT_BUFFER ) {
            yyensure_buffer_stack (yyscanner);
            YY_CURRENT_BUFFER_LVALUE =
                yy_create_buffer(yyin,YY_BUF_SIZE ,yyscanner);
        }

        yy_load_buffer_state(yyscanner );
        }

    while ( 1 )     /* loops until end-of-file is reached */
        {
/* %% [8.0] yymore()-related code goes here */
        yy_cp = yyg->yy_c_buf_p;

        /* Support of yytext. */
        *yy_cp = yyg->yy_hold_char;

        /* yy_bp points to the position in yy_ch_buf of the start of
         * the current run.
         */
        yy_bp = yy_cp;

/* %% [9.0] code to set up and find next match goes here */
        yy_current_state = yyg->yy_start;
yy_match:
        do
            {
            register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
            if ( yy_accept[yy_current_state] )
                {
                yyg->yy_last_accepting_state = yy_current_state;
                yyg->yy_last_accepting_cpos = yy_cp;
                }
            while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
                {
                yy_current_state = (int) yy_def[yy_current_state];
                if ( yy_current_state >= 391 )
                    yy_c = yy_meta[(unsigned int) yy_c];
                }
            yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
            ++yy_cp;
            }
        while ( yy_current_state != 390 );
        yy_cp = yyg->yy_last_accepting_cpos;
        yy_current_state = yyg->yy_last_accepting_state;

yy_find_action:
/* %% [10.0] code to find the action number goes here */
        yy_act = yy_accept[yy_current_state];

        YY_DO_BEFORE_ACTION;

/* %% [11.0] code for yylineno update goes here */

do_action:  /* This label is used only to access EOF actions. */

/* %% [12.0] debug code goes here */
        if ( yy_flex_debug )
            {
            if ( yy_act == 0 )
                fprintf( stderr, "--scanner backing up\n" );
            else if ( yy_act < 117 )
                fprintf( stderr, "--accepting rule at line %ld (\"%s\")\n",
                         (long)yy_rule_linenum[yy_act], yytext );
            else if ( yy_act == 117 )
                fprintf( stderr, "--accepting default rule (\"%s\")\n",
                         yytext );
            else if ( yy_act == 118 )
                fprintf( stderr, "--(end of buffer or a NUL)\n" );
            else
                fprintf( stderr, "--EOF (start condition %d)\n", YY_START );
            }

        switch ( yy_act )
    { /* beginning of action switch */
/* %% [13.0] actions go here */
            case 0: /* must back up */
            /* undo the effects of YY_DO_BEFORE_ACTION */
            *yy_cp = yyg->yy_hold_char;
            yy_cp = yyg->yy_last_accepting_cpos;
            yy_current_state = yyg->yy_last_accepting_state;
            goto yy_find_action;

case 1:
YY_RULE_SETUP
#line 271 "pir.l"
{ /* ignore whitespace */ }
    YY_BREAK
case 2:
/* rule 2 can match eol */
YY_RULE_SETUP
#line 273 "pir.l"
{ /* a set of continuous newlines yields a single newline token. */
                   return TK_NL;
                 }
    YY_BREAK
case 3:
YY_RULE_SETUP
#line 277 "pir.l"
{ return TK_ASSIGN_USHIFT; }
    YY_BREAK
case 4:
YY_RULE_SETUP
#line 278 "pir.l"
{ return TK_USHIFT; }
    YY_BREAK
case 5:
YY_RULE_SETUP
#line 279 "pir.l"
{ return TK_ASSIGN_RSHIFT; }
    YY_BREAK
case 6:
YY_RULE_SETUP
#line 280 "pir.l"
{ return TK_RSHIFT; }
    YY_BREAK
case 7:
YY_RULE_SETUP
#line 281 "pir.l"
{ return TK_LSHIFT; }
    YY_BREAK
case 8:
YY_RULE_SETUP
#line 282 "pir.l"
{ return TK_ARROW; }
    YY_BREAK
case 9:
YY_RULE_SETUP
#line 283 "pir.l"
{ return TK_EQ; }
    YY_BREAK
case 10:
YY_RULE_SETUP
#line 284 "pir.l"
{ return TK_NE; }
    YY_BREAK
case 11:
YY_RULE_SETUP
#line 285 "pir.l"
{ return TK_LE; }
    YY_BREAK
case 12:
YY_RULE_SETUP
#line 286 "pir.l"
{ return TK_GE; }
    YY_BREAK
case 13:
YY_RULE_SETUP
#line 287 "pir.l"
{ return TK_LT; }
    YY_BREAK
case 14:
YY_RULE_SETUP
#line 288 "pir.l"
{ return TK_GT; }
    YY_BREAK
case 15:
YY_RULE_SETUP
#line 290 "pir.l"
{ return TK_FDIV; }
    YY_BREAK
case 16:
YY_RULE_SETUP
#line 291 "pir.l"
{ return TK_AND; }
    YY_BREAK
case 17:
YY_RULE_SETUP
#line 292 "pir.l"
{ return TK_OR; }
    YY_BREAK
case 18:
YY_RULE_SETUP
#line 293 "pir.l"
{ return TK_XOR; }
    YY_BREAK
case 19:
YY_RULE_SETUP
#line 295 "pir.l"
{ return '+'; }
    YY_BREAK
case 20:
YY_RULE_SETUP
#line 296 "pir.l"
{ return '%'; }
    YY_BREAK
case 21:
YY_RULE_SETUP
#line 297 "pir.l"
{ return '*'; }
    YY_BREAK
case 22:
YY_RULE_SETUP
#line 298 "pir.l"
{ return '/'; }
    YY_BREAK
case 23:
YY_RULE_SETUP
#line 299 "pir.l"
{ return '!'; }
    YY_BREAK
case 24:
YY_RULE_SETUP
#line 300 "pir.l"
{ return '~'; }
    YY_BREAK
case 25:
YY_RULE_SETUP
#line 301 "pir.l"
{ return '-'; }
    YY_BREAK
case 26:
YY_RULE_SETUP
#line 302 "pir.l"
{ return '('; }
    YY_BREAK
case 27:
YY_RULE_SETUP
#line 303 "pir.l"
{ return ')'; }
    YY_BREAK
case 28:
YY_RULE_SETUP
#line 304 "pir.l"
{ return ','; }
    YY_BREAK
case 29:
YY_RULE_SETUP
#line 305 "pir.l"
{ return '['; }
    YY_BREAK
case 30:
YY_RULE_SETUP
#line 306 "pir.l"
{ return ']'; }
    YY_BREAK
case 31:
YY_RULE_SETUP
#line 308 "pir.l"
{ /* if the dot is surrounded by whitespace, it's a concatenation operator */
              return TK_CONC;
            }
    YY_BREAK
case 32:
YY_RULE_SETUP
#line 311 "pir.l"
{ return '.'; }
    YY_BREAK
case 33:
YY_RULE_SETUP
#line 312 "pir.l"
{ return '='; }
    YY_BREAK
case 34:
YY_RULE_SETUP
#line 313 "pir.l"
{ return ';'; }
    YY_BREAK
case 35:
YY_RULE_SETUP
#line 315 "pir.l"
{ return TK_ASSIGN_INC; }
    YY_BREAK
case 36:
YY_RULE_SETUP
#line 316 "pir.l"
{ return TK_ASSIGN_DEC; }
    YY_BREAK
case 37:
YY_RULE_SETUP
#line 317 "pir.l"
{ return TK_ASSIGN_DIV; }
    YY_BREAK
case 38:
YY_RULE_SETUP
#line 318 "pir.l"
{ return TK_ASSIGN_MUL; }
    YY_BREAK
case 39:
YY_RULE_SETUP
#line 319 "pir.l"
{ return TK_ASSIGN_MOD; }
    YY_BREAK
case 40:
YY_RULE_SETUP
#line 320 "pir.l"
{ return TK_ASSIGN_POW; }
    YY_BREAK
case 41:
YY_RULE_SETUP
#line 321 "pir.l"
{ return TK_ASSIGN_BOR; }
    YY_BREAK
case 42:
YY_RULE_SETUP
#line 322 "pir.l"
{ return TK_ASSIGN_BAND; }
    YY_BREAK
case 43:
YY_RULE_SETUP
#line 323 "pir.l"
{ return TK_ASSIGN_FDIV; }
    YY_BREAK
case 44:
YY_RULE_SETUP
#line 324 "pir.l"
{ return TK_ASSIGN_BNOT; }
    YY_BREAK
case 45:
YY_RULE_SETUP
#line 325 "pir.l"
{ return TK_ASSIGN_CONC; }
    YY_BREAK
case 46:
YY_RULE_SETUP
#line 327 "pir.l"
{ return TK_IF; }
    YY_BREAK
case 47:
YY_RULE_SETUP
#line 328 "pir.l"
{ return TK_GOTO; }
    YY_BREAK
case 48:
YY_RULE_SETUP
#line 329 "pir.l"
{ return TK_N_OPERATORS; }
    YY_BREAK
case 49:
YY_RULE_SETUP
#line 330 "pir.l"
{ return TK_UNLESS; }
    YY_BREAK
case 50:
YY_RULE_SETUP
#line 331 "pir.l"
{ return TK_NULL; }
    YY_BREAK
case 51:
YY_RULE_SETUP
#line 333 "pir.l"
{ return TK_INT; }
    YY_BREAK
case 52:
YY_RULE_SETUP
#line 334 "pir.l"
{ return TK_NUM; }
    YY_BREAK
case 53:
YY_RULE_SETUP
#line 335 "pir.l"
{ return TK_PMC; }
    YY_BREAK
case 54:
YY_RULE_SETUP
#line 336 "pir.l"
{ return TK_STRING; }
    YY_BREAK
case 55:
YY_RULE_SETUP
#line 338 "pir.l"
{ return TK_ARG; }
    YY_BREAK
case 56:
YY_RULE_SETUP
#line 339 "pir.l"
{ return TK_CONST; }
    YY_BREAK
case 57:
YY_RULE_SETUP
#line 340 "pir.l"
{ return TK_END; }
    YY_BREAK
case 58:
YY_RULE_SETUP
#line 342 "pir.l"
{ return TK_GET_RESULTS; }
    YY_BREAK
case 59:
YY_RULE_SETUP
#line 343 "pir.l"
{ return TK_GLOBALCONST; }
    YY_BREAK
case 60:
YY_RULE_SETUP
#line 344 "pir.l"
{ return TK_HLL; }
    YY_BREAK
case 61:
YY_RULE_SETUP
#line 345 "pir.l"
{ return TK_HLL_MAP; }
    YY_BREAK
case 62:
YY_RULE_SETUP
#line 346 "pir.l"
{ return TK_INVOCANT; }
    YY_BREAK
case 63:
YY_RULE_SETUP
#line 347 "pir.l"
{ return TK_LEX; }
    YY_BREAK
case 64:
YY_RULE_SETUP
#line 348 "pir.l"
{ return TK_LOADLIB; }
    YY_BREAK
case 65:
YY_RULE_SETUP
#line 349 "pir.l"
{ return TK_LOCAL; }
    YY_BREAK
case 66:
YY_RULE_SETUP
#line 351 "pir.l"
{ return TK_METH_CALL; }
    YY_BREAK
case 67:
YY_RULE_SETUP
#line 352 "pir.l"
{ return TK_NAMESPACE; }
    YY_BREAK
case 68:
YY_RULE_SETUP
#line 353 "pir.l"
{ return TK_NCI_CALL; }
    YY_BREAK
case 69:
YY_RULE_SETUP
#line 354 "pir.l"
{ return TK_PARAM; }
    YY_BREAK
case 70:
YY_RULE_SETUP
#line 355 "pir.l"
{ return TK_BEGIN_CALL; }
    YY_BREAK
case 71:
YY_RULE_SETUP
#line 356 "pir.l"
{ return TK_BEGIN_RETURN; }
    YY_BREAK
case 72:
YY_RULE_SETUP
#line 357 "pir.l"
{ return TK_BEGIN_YIELD; }
    YY_BREAK
case 73:
YY_RULE_SETUP
#line 358 "pir.l"
{ return TK_CALL; }
    YY_BREAK
case 74:
YY_RULE_SETUP
#line 359 "pir.l"
{ return TK_END_CALL; }
    YY_BREAK
case 75:
YY_RULE_SETUP
#line 360 "pir.l"
{ return TK_END_RETURN; }
    YY_BREAK
case 76:
YY_RULE_SETUP
#line 361 "pir.l"
{ return TK_END_YIELD; }
    YY_BREAK
case 77:
YY_RULE_SETUP
#line 362 "pir.l"
{ return TK_PRAGMA; }
    YY_BREAK
case 78:
YY_RULE_SETUP
#line 363 "pir.l"
{ return TK_RESULT; }
    YY_BREAK
case 79:
YY_RULE_SETUP
#line 364 "pir.l"
{ return TK_RETURN; }
    YY_BREAK
case 80:
YY_RULE_SETUP
#line 365 "pir.l"
{ return TK_SUB; }
    YY_BREAK
case 81:
YY_RULE_SETUP
#line 366 "pir.l"
{ return TK_YIELD; }
    YY_BREAK
case 82:
YY_RULE_SETUP
#line 368 "pir.l"
{ return TK_FLAG_ANON; }
    YY_BREAK
case 83:
YY_RULE_SETUP
#line 369 "pir.l"
{ return TK_FLAG_INIT; }
    YY_BREAK
case 84:
YY_RULE_SETUP
#line 370 "pir.l"
{ return TK_FLAG_LOAD; }
    YY_BREAK
case 85:
YY_RULE_SETUP
#line 371 "pir.l"
{ return TK_FLAG_POSTCOMP; }
    YY_BREAK
case 86:
YY_RULE_SETUP
#line 372 "pir.l"
{ return TK_FLAG_IMMEDIATE; }
    YY_BREAK
case 87:
YY_RULE_SETUP
#line 373 "pir.l"
{ return TK_FLAG_MAIN; }
    YY_BREAK
case 88:
YY_RULE_SETUP
#line 374 "pir.l"
{ return TK_FLAG_METHOD; }
    YY_BREAK
case 89:
YY_RULE_SETUP
#line 375 "pir.l"
{ return TK_FLAG_LEX; }
    YY_BREAK
case 90:
YY_RULE_SETUP
#line 376 "pir.l"
{ return TK_FLAG_OUTER; }
    YY_BREAK
case 91:
YY_RULE_SETUP
#line 377 "pir.l"
{ return TK_FLAG_VTABLE; }
    YY_BREAK
case 92:
YY_RULE_SETUP
#line 378 "pir.l"
{ return TK_FLAG_MULTI; }
    YY_BREAK
case 93:
YY_RULE_SETUP
#line 380 "pir.l"
{ return TK_FLAG_UNIQUE_REG; }
    YY_BREAK
case 94:
YY_RULE_SETUP
#line 381 "pir.l"
{ return TK_FLAG_OPTIONAL; }
    YY_BREAK
case 95:
YY_RULE_SETUP
#line 382 "pir.l"
{ return TK_FLAG_OPT_FLAG; }
    YY_BREAK
case 96:
YY_RULE_SETUP
#line 383 "pir.l"
{ return TK_FLAG_SLURPY; }
    YY_BREAK
case 97:
YY_RULE_SETUP
#line 384 "pir.l"
{ return TK_FLAG_NAMED; }
    YY_BREAK
case 98:
YY_RULE_SETUP
#line 385 "pir.l"
{ return TK_FLAG_FLAT; }
    YY_BREAK
case 99:
YY_RULE_SETUP
#line 386 "pir.l"
{ return TK_FLAG_INVOCANT; }
    YY_BREAK
case 100:
YY_RULE_SETUP
#line 389 "pir.l"
{ /* copy the string, remove the quotes. */
               yylval->sval = dupstrn(yytext + 1, yyleng - 2);
               return TK_STRINGC;
             }
    YY_BREAK
case 101:
YY_RULE_SETUP
#line 394 "pir.l"
{ yylval->ival = atoi(yytext + 1); return TK_PASM_PREG; }
    YY_BREAK
case 102:
YY_RULE_SETUP
#line 395 "pir.l"
{ yylval->ival = atoi(yytext + 1); return TK_PASM_SREG; }
    YY_BREAK
case 103:
YY_RULE_SETUP
#line 396 "pir.l"
{ yylval->ival = atoi(yytext + 1); return TK_PASM_NREG; }
    YY_BREAK
case 104:
YY_RULE_SETUP
#line 397 "pir.l"
{ yylval->ival = atoi(yytext + 1); return TK_PASM_IREG; }
    YY_BREAK
case 105:
YY_RULE_SETUP
#line 399 "pir.l"
{ yylval->ival = atoi(yytext + 2); return TK_SYM_PREG; }
    YY_BREAK
case 106:
YY_RULE_SETUP
#line 400 "pir.l"
{ yylval->ival = atoi(yytext + 2); return TK_SYM_SREG; }
    YY_BREAK
case 107:
YY_RULE_SETUP
#line 401 "pir.l"
{ yylval->ival = atoi(yytext + 2); return TK_SYM_NREG; }
    YY_BREAK
case 108:
YY_RULE_SETUP
#line 402 "pir.l"
{ yylval->ival = atoi(yytext + 2); return TK_SYM_IREG; }
    YY_BREAK
case 109:
YY_RULE_SETUP
#line 404 "pir.l"
{ /* make the label Id available in the parser. remove the ":" first. */
                yylval->sval = dupstrn(yytext, yyleng - 1);
                return TK_LABEL;
              }
    YY_BREAK
case 110:
YY_RULE_SETUP
#line 409 "pir.l"
{ /* make the Id or op available in the parser. */
                yylval->sval = dupstr(yytext);
                return is_parrot_op(yytext) ? TK_PARROT_OP : TK_IDENT;
              }
    YY_BREAK
case 111:
YY_RULE_SETUP
#line 415 "pir.l"
{ yylval->dval = atof(yytext); return TK_NUMC; }
    YY_BREAK
case 112:
YY_RULE_SETUP
#line 416 "pir.l"
{ yylval->ival = atoi(yytext); return TK_INTC; }
    YY_BREAK
case 113:
YY_RULE_SETUP
#line 417 "pir.l"
{ yylval->ival = atoi(yytext); return TK_INTC; }
    YY_BREAK
case 114:
YY_RULE_SETUP
#line 418 "pir.l"
{ yylval->ival = atoi(yytext); return TK_INTC; }
    YY_BREAK
case 115:
YY_RULE_SETUP
#line 419 "pir.l"
{ yylval->ival = atoi(yytext); return TK_INTC; }
    YY_BREAK
case 116:
YY_RULE_SETUP
#line 424 "pir.l"
{ /* any character not covered in the rules above is an error. */
         lexer_state *my_lexer = yyget_extra(yyscanner);
         syntax_error(yyscanner, my_lexer, "Unexpected character");
       }
    YY_BREAK
case YY_STATE_EOF(INITIAL):
#line 430 "pir.l"
{ /* end of file, stop scanning. */
              yyterminate();
            }
    YY_BREAK
case 117:
YY_RULE_SETUP
#line 434 "pir.l"
ECHO;
    YY_BREAK
#line 2050 "pirlexer.c"

    case YY_END_OF_BUFFER:
        {
        /* Amount of text matched not including the EOB char. */
        int yy_amount_of_matched_text = (int) (yy_cp - yyg->yytext_ptr) - 1;

        /* Undo the effects of YY_DO_BEFORE_ACTION. */
        *yy_cp = yyg->yy_hold_char;
        YY_RESTORE_YY_MORE_OFFSET

        if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
            {
            /* We're scanning a new file or input source.  It's
             * possible that this happened because the user
             * just pointed yyin at a new source and called
             * yylex().  If so, then we have to assure
             * consistency between YY_CURRENT_BUFFER and our
             * globals.  Here is the right place to do so, because
             * this is the first action (other than possibly a
             * back-up) that will match for the new input source.
             */
            yyg->yy_n_chars = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
            YY_CURRENT_BUFFER_LVALUE->yy_input_file = yyin;
            YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
            }

        /* Note that here we test for yy_c_buf_p "<=" to the position
         * of the first EOB in the buffer, since yy_c_buf_p will
         * already have been incremented past the NUL character
         * (since all states make transitions on EOB to the
         * end-of-buffer state).  Contrast this with the test
         * in input().
         */
        if ( yyg->yy_c_buf_p <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] )
            { /* This was really a NUL. */
            yy_state_type yy_next_state;

            yyg->yy_c_buf_p = yyg->yytext_ptr + yy_amount_of_matched_text;

            yy_current_state = yy_get_previous_state( yyscanner );

            /* Okay, we're now positioned to make the NUL
             * transition.  We couldn't have
             * yy_get_previous_state() go ahead and do it
             * for us because it doesn't know how to deal
             * with the possibility of jamming (and we don't
             * want to build jamming into it because then it
             * will run more slowly).
             */

            yy_next_state = yy_try_NUL_trans( yy_current_state , yyscanner);

            yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;

            if ( yy_next_state )
                {
                /* Consume the NUL. */
                yy_cp = ++yyg->yy_c_buf_p;
                yy_current_state = yy_next_state;
                goto yy_match;
                }

            else
                {
/* %% [14.0] code to do back-up for compressed tables and set up yy_cp goes here */
                yy_cp = yyg->yy_last_accepting_cpos;
                yy_current_state = yyg->yy_last_accepting_state;
                goto yy_find_action;
                }
            }

        else switch ( yy_get_next_buffer( yyscanner ) )
            {
            case EOB_ACT_END_OF_FILE:
                {
                yyg->yy_did_buffer_switch_on_eof = 0;

                if ( yywrap(yyscanner ) )
                    {
                    /* Note: because we've taken care in
                     * yy_get_next_buffer() to have set up
                     * yytext, we can now set up
                     * yy_c_buf_p so that if some total
                     * hoser (like flex itself) wants to
                     * call the scanner after we return the
                     * YY_NULL, it'll still work - another
                     * YY_NULL will get returned.
                     */
                    yyg->yy_c_buf_p = yyg->yytext_ptr + YY_MORE_ADJ;

                    yy_act = YY_STATE_EOF(YY_START);
                    goto do_action;
                    }

                else
                    {
                    if ( ! yyg->yy_did_buffer_switch_on_eof )
                        YY_NEW_FILE;
                    }
                break;
                }

            case EOB_ACT_CONTINUE_SCAN:
                yyg->yy_c_buf_p =
                    yyg->yytext_ptr + yy_amount_of_matched_text;

                yy_current_state = yy_get_previous_state( yyscanner );

                yy_cp = yyg->yy_c_buf_p;
                yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;
                goto yy_match;

            case EOB_ACT_LAST_MATCH:
                yyg->yy_c_buf_p =
                &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars];

                yy_current_state = yy_get_previous_state( yyscanner );

                yy_cp = yyg->yy_c_buf_p;
                yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;
                goto yy_find_action;
            }
        break;
        }

    default:
        YY_FATAL_ERROR(
            "fatal flex scanner internal error--no action found" );
    } /* end of action switch */
        } /* end of scanning one token */
} /* end of yylex */
/* %ok-for-header */

/* %if-c++-only */
/* %not-for-header */

/* %ok-for-header */

/* %endif */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *  EOB_ACT_LAST_MATCH -
 *  EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *  EOB_ACT_END_OF_FILE - end of file
 */
/* %if-c-only */
static int yy_get_next_buffer (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    register char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
    register char *source = yyg->yytext_ptr;
    register int number_to_move, i;
    int ret_val;

    if ( yyg->yy_c_buf_p > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars + 1] )
        YY_FATAL_ERROR(
        "fatal flex scanner internal error--end of buffer missed" );

    if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
        { /* Don't try to fill the buffer, so this is an EOF. */
        if ( yyg->yy_c_buf_p - yyg->yytext_ptr - YY_MORE_ADJ == 1 )
            {
            /* We matched a single character, the EOB, so
             * treat this as a final EOF.
             */
            return EOB_ACT_END_OF_FILE;
            }

        else
            {
            /* We matched some text prior to the EOB, first
             * process it.
             */
            return EOB_ACT_LAST_MATCH;
            }
        }

    /* Try to read more data. */

    /* First move last chars to start of buffer. */
    number_to_move = (int) (yyg->yy_c_buf_p - yyg->yytext_ptr) - 1;

    for ( i = 0; i < number_to_move; ++i )
        *(dest++) = *(source++);

    if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
        /* don't do the read, it's not guaranteed to return an EOF,
         * just force an EOF
         */
        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars = 0;

    else
        {
            int num_to_read =
            YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

        while ( num_to_read <= 0 )
            { /* Not enough room in the buffer - grow it. */

            /* just a shorter name for the current buffer */
            YY_BUFFER_STATE b = YY_CURRENT_BUFFER;

            int yy_c_buf_p_offset =
                (int) (yyg->yy_c_buf_p - b->yy_ch_buf);

            if ( b->yy_is_our_buffer )
                {
                int new_size = b->yy_buf_size * 2;

                if ( new_size <= 0 )
                    b->yy_buf_size += b->yy_buf_size / 8;
                else
                    b->yy_buf_size *= 2;

                b->yy_ch_buf = (char *)
                    /* Include room in for 2 EOB chars. */
                    yyrealloc((void *) b->yy_ch_buf,b->yy_buf_size + 2 ,yyscanner );
                }
            else
                /* Can't grow it, we don't own it. */
                b->yy_ch_buf = 0;

            if ( ! b->yy_ch_buf )
                YY_FATAL_ERROR(
                "fatal error - scanner input buffer overflow" );

            yyg->yy_c_buf_p = &b->yy_ch_buf[yy_c_buf_p_offset];

            num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
                        number_to_move - 1;

            }

        if ( num_to_read > YY_READ_BUF_SIZE )
            num_to_read = YY_READ_BUF_SIZE;

        /* Read in more data. */
        YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
            yyg->yy_n_chars, num_to_read );

        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
        }

    if ( yyg->yy_n_chars == 0 )
        {
        if ( number_to_move == YY_MORE_ADJ )
            {
            ret_val = EOB_ACT_END_OF_FILE;
            yyrestart(yyin  ,yyscanner);
            }

        else
            {
            ret_val = EOB_ACT_LAST_MATCH;
            YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
                YY_BUFFER_EOF_PENDING;
            }
        }

    else
        ret_val = EOB_ACT_CONTINUE_SCAN;

    yyg->yy_n_chars += number_to_move;
    YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] = YY_END_OF_BUFFER_CHAR;
    YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars + 1] = YY_END_OF_BUFFER_CHAR;

    yyg->yytext_ptr = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

    return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

/* %if-c-only */
/* %not-for-header */

    static yy_state_type yy_get_previous_state (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    register yy_state_type yy_current_state;
    register char *yy_cp;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

/* %% [15.0] code to get the start state into yy_current_state goes here */
    yy_current_state = yyg->yy_start;

    for ( yy_cp = yyg->yytext_ptr + YY_MORE_ADJ; yy_cp < yyg->yy_c_buf_p; ++yy_cp )
        {
/* %% [16.0] code to find the next state goes here */
        register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
        if ( yy_accept[yy_current_state] )
            {
            yyg->yy_last_accepting_state = yy_current_state;
            yyg->yy_last_accepting_cpos = yy_cp;
            }
        while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
            {
            yy_current_state = (int) yy_def[yy_current_state];
            if ( yy_current_state >= 391 )
                yy_c = yy_meta[(unsigned int) yy_c];
            }
        yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
        }

    return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *  next_state = yy_try_NUL_trans( current_state );
 */
/* %if-c-only */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    register int yy_is_jam;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner; /* This var may be unused depending upon options. */
/* %% [17.0] code to find the next state, and perhaps do backing up, goes here */
    register char *yy_cp = yyg->yy_c_buf_p;

    register YY_CHAR yy_c = 1;
    if ( yy_accept[yy_current_state] )
        {
        yyg->yy_last_accepting_state = yy_current_state;
        yyg->yy_last_accepting_cpos = yy_cp;
        }
    while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
        {
        yy_current_state = (int) yy_def[yy_current_state];
        if ( yy_current_state >= 391 )
            yy_c = yy_meta[(unsigned int) yy_c];
        }
    yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
    yy_is_jam = (yy_current_state == 390);

    return yy_is_jam ? 0 : yy_current_state;
}

/* %if-c-only */

/* %endif */

/* %if-c-only */
#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (yyscan_t yyscanner)
#else
    static int input  (yyscan_t yyscanner)
#endif

/* %endif */
/* %if-c++-only */
/* %endif */
{
    int c;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    *yyg->yy_c_buf_p = yyg->yy_hold_char;

    if ( *yyg->yy_c_buf_p == YY_END_OF_BUFFER_CHAR )
        {
        /* yy_c_buf_p now points to the character we want to return.
         * If this occurs *before* the EOB characters, then it's a
         * valid NUL; if not, then we've hit the end of the buffer.
         */
        if ( yyg->yy_c_buf_p < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] )
            /* This was really a NUL. */
            *yyg->yy_c_buf_p = '\0';

        else
            { /* need more input */
            int offset = yyg->yy_c_buf_p - yyg->yytext_ptr;
            ++yyg->yy_c_buf_p;

            switch ( yy_get_next_buffer( yyscanner ) )
                {
                case EOB_ACT_LAST_MATCH:
                    /* This happens because yy_g_n_b()
                     * sees that we've accumulated a
                     * token and flags that we need to
                     * try matching the token before
                     * proceeding.  But for input(),
                     * there's no matching to consider.
                     * So convert the EOB_ACT_LAST_MATCH
                     * to EOB_ACT_END_OF_FILE.
                     */

                    /* Reset buffer status. */
                    yyrestart(yyin ,yyscanner);

                    /*FALLTHROUGH*/

                case EOB_ACT_END_OF_FILE:
                    {
                    if ( yywrap(yyscanner ) )
                        return EOF;

                    if ( ! yyg->yy_did_buffer_switch_on_eof )
                        YY_NEW_FILE;
#ifdef __cplusplus
                    return yyinput(yyscanner);
#else
                    return input(yyscanner);
#endif
                    }

                case EOB_ACT_CONTINUE_SCAN:
                    yyg->yy_c_buf_p = yyg->yytext_ptr + offset;
                    break;
                }
            }
        }

    c = *(unsigned char *) yyg->yy_c_buf_p; /* cast for 8-bit char's */
    *yyg->yy_c_buf_p = '\0';    /* preserve yytext */
    yyg->yy_hold_char = *++yyg->yy_c_buf_p;

/* %% [19.0] update BOL and yylineno */

    return c;
}
/* %if-c-only */
#endif  /* ifndef YY_NO_INPUT */
/* %endif */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * @param yyscanner The scanner object.
 * @note This function does not reset the start condition to @c INITIAL .
 */
/* %if-c-only */
    void yyrestart  (FILE * input_file , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    if ( ! YY_CURRENT_BUFFER ){
        yyensure_buffer_stack (yyscanner);
        YY_CURRENT_BUFFER_LVALUE =
            yy_create_buffer(yyin,YY_BUF_SIZE ,yyscanner);
    }

    yy_init_buffer(YY_CURRENT_BUFFER,input_file ,yyscanner);
    yy_load_buffer_state(yyscanner );
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * @param yyscanner The scanner object.
 */
/* %if-c-only */
    void yy_switch_to_buffer  (YY_BUFFER_STATE  new_buffer , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    /* TODO. We should be able to replace this entire function body
     * with
     *      yypop_buffer_state();
     *      yypush_buffer_state(new_buffer);
     */
    yyensure_buffer_stack (yyscanner);
    if ( YY_CURRENT_BUFFER == new_buffer )
        return;

    if ( YY_CURRENT_BUFFER )
        {
        /* Flush out information for old buffer. */
        *yyg->yy_c_buf_p = yyg->yy_hold_char;
        YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = yyg->yy_c_buf_p;
        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
        }

    YY_CURRENT_BUFFER_LVALUE = new_buffer;
    yy_load_buffer_state(yyscanner );

    /* We don't actually know whether we did this switch during
     * EOF (yywrap()) processing, but the only time this flag
     * is looked at is after yywrap() is called, so it's safe
     * to go ahead and always set it.
     */
    yyg->yy_did_buffer_switch_on_eof = 1;
}

/* %if-c-only */
static void yy_load_buffer_state  (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyg->yy_n_chars = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
    yyg->yytext_ptr = yyg->yy_c_buf_p = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
    yyin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
    yyg->yy_hold_char = *yyg->yy_c_buf_p;
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * @param yyscanner The scanner object.
 * @return the allocated buffer state.
 */
/* %if-c-only */
    YY_BUFFER_STATE yy_create_buffer  (FILE * file, int  size , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    YY_BUFFER_STATE b;

    b = (YY_BUFFER_STATE) yyalloc(sizeof( struct yy_buffer_state ) ,yyscanner );
    if ( ! b )
        YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

    b->yy_buf_size = size;

    /* yy_ch_buf has to be 2 characters longer than the size given because
     * we need to put in 2 end-of-buffer characters.
     */
    b->yy_ch_buf = (char *) yyalloc(b->yy_buf_size + 2 ,yyscanner );
    if ( ! b->yy_ch_buf )
        YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

    b->yy_is_our_buffer = 1;

    yy_init_buffer(b,file ,yyscanner);

    return b;
}

/** Destroy the buffer.
 * @param b a buffer created with yy_create_buffer()
 * @param yyscanner The scanner object.
 */
/* %if-c-only */
    void yy_delete_buffer (YY_BUFFER_STATE  b , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    if ( ! b )
        return;

    if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
        YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

    if ( b->yy_is_our_buffer )
        yyfree((void *) b->yy_ch_buf ,yyscanner );

    yyfree((void *) b ,yyscanner );
}

/* %if-c-only */

/* %endif */

/* %if-c++-only */
/* %endif */

/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a yyrestart() or at EOF.
 */
/* %if-c-only */
    static void yy_init_buffer  (YY_BUFFER_STATE  b, FILE * file , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */

{
    int oerrno = errno;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    yy_flush_buffer(b ,yyscanner);

    b->yy_input_file = file;
    b->yy_fill_buffer = 1;

    /* If b is the current buffer, then yy_init_buffer was _probably_
     * called from yyrestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

/* %if-c-only */

        b->yy_is_interactive = 0;

/* %endif */
/* %if-c++-only */
/* %endif */
    errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * @param yyscanner The scanner object.
 */
/* %if-c-only */
    void yy_flush_buffer (YY_BUFFER_STATE  b , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    if ( ! b )
        return;

    b->yy_n_chars = 0;

    /* We always need two end-of-buffer characters.  The first causes
     * a transition to the end-of-buffer state.  The second causes
     * a jam in that state.
     */
    b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
    b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

    b->yy_buf_pos = &b->yy_ch_buf[0];

    b->yy_at_bol = 1;
    b->yy_buffer_status = YY_BUFFER_NEW;

    if ( b == YY_CURRENT_BUFFER )
        yy_load_buffer_state(yyscanner );
}

/* %if-c-or-c++ */
/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  @param yyscanner The scanner object.
 */
/* %if-c-only */
void yypush_buffer_state (YY_BUFFER_STATE new_buffer , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    if (new_buffer == NULL)
        return;

    yyensure_buffer_stack(yyscanner);

    /* This block is copied from yy_switch_to_buffer. */
    if ( YY_CURRENT_BUFFER )
        {
        /* Flush out information for old buffer. */
        *yyg->yy_c_buf_p = yyg->yy_hold_char;
        YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = yyg->yy_c_buf_p;
        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
        }

    /* Only push if top exists. Otherwise, replace top. */
    if (YY_CURRENT_BUFFER)
        yyg->yy_buffer_stack_top++;
    YY_CURRENT_BUFFER_LVALUE = new_buffer;

    /* copied from yy_switch_to_buffer. */
    yy_load_buffer_state(yyscanner );
    yyg->yy_did_buffer_switch_on_eof = 1;
}
/* %endif */

/* %if-c-or-c++ */
/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  @param yyscanner The scanner object.
 */
/* %if-c-only */
void yypop_buffer_state (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    if (!YY_CURRENT_BUFFER)
        return;

    yy_delete_buffer(YY_CURRENT_BUFFER ,yyscanner);
    YY_CURRENT_BUFFER_LVALUE = NULL;
    if (yyg->yy_buffer_stack_top > 0)
        --yyg->yy_buffer_stack_top;

    if (YY_CURRENT_BUFFER) {
        yy_load_buffer_state(yyscanner );
        yyg->yy_did_buffer_switch_on_eof = 1;
    }
}
/* %endif */

/* %if-c-or-c++ */
/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
/* %if-c-only */
static void yyensure_buffer_stack (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    int num_to_alloc;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    if (!yyg->yy_buffer_stack) {

        /* First allocation is just for 2 elements, since we don't know if this
         * scanner will even need a stack. We use 2 instead of 1 to avoid an
         * immediate realloc on the next call.
         */
        num_to_alloc = 1;
        yyg->yy_buffer_stack = (struct yy_buffer_state**)yyalloc
                                (num_to_alloc * sizeof(struct yy_buffer_state*)
                                , yyscanner);

        memset(yyg->yy_buffer_stack, 0, num_to_alloc * sizeof(struct yy_buffer_state*));

        yyg->yy_buffer_stack_max = num_to_alloc;
        yyg->yy_buffer_stack_top = 0;
        return;
    }

    if (yyg->yy_buffer_stack_top >= (yyg->yy_buffer_stack_max) - 1){

        /* Increase the buffer to prepare for a possible push. */
        int grow_size = 8 /* arbitrary grow size */;

        num_to_alloc = yyg->yy_buffer_stack_max + grow_size;
        yyg->yy_buffer_stack = (struct yy_buffer_state**)yyrealloc
                                (yyg->yy_buffer_stack,
                                num_to_alloc * sizeof(struct yy_buffer_state*)
                                , yyscanner);

        /* zero only the new slots.*/
        memset(yyg->yy_buffer_stack + yyg->yy_buffer_stack_max, 0, grow_size * sizeof(struct yy_buffer_state*));
        yyg->yy_buffer_stack_max = num_to_alloc;
    }
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE yy_scan_buffer  (char * base, yy_size_t  size , yyscan_t yyscanner)
{
    YY_BUFFER_STATE b;

    if ( size < 2 ||
         base[size-2] != YY_END_OF_BUFFER_CHAR ||
         base[size-1] != YY_END_OF_BUFFER_CHAR )
        /* They forgot to leave room for the EOB's. */
        return 0;

    b = (YY_BUFFER_STATE) yyalloc(sizeof( struct yy_buffer_state ) ,yyscanner );
    if ( ! b )
        YY_FATAL_ERROR( "out of dynamic memory in yy_scan_buffer()" );

    b->yy_buf_size = size - 2;  /* "- 2" to take care of EOB's */
    b->yy_buf_pos = b->yy_ch_buf = base;
    b->yy_is_our_buffer = 0;
    b->yy_input_file = 0;
    b->yy_n_chars = b->yy_buf_size;
    b->yy_is_interactive = 0;
    b->yy_at_bol = 1;
    b->yy_fill_buffer = 0;
    b->yy_buffer_status = YY_BUFFER_NEW;

    yy_switch_to_buffer(b ,yyscanner );

    return b;
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan a string. The next call to yylex() will
 * scan from a @e copy of @a str.
 * @param str a NUL-terminated string to scan
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       yy_scan_bytes() instead.
 */
YY_BUFFER_STATE yy_scan_string (yyconst char * yystr , yyscan_t yyscanner)
{

    return yy_scan_bytes(yystr,strlen(yystr) ,yyscanner);
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan the given bytes. The next call to yylex() will
 * scan from a @e copy of @a bytes.
 * @param bytes the byte buffer to scan
 * @param len the number of bytes in the buffer pointed to by @a bytes.
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE yy_scan_bytes  (yyconst char * yybytes, int  _yybytes_len , yyscan_t yyscanner)
{
    YY_BUFFER_STATE b;
    char *buf;
    yy_size_t n;
    int i;

    /* Get memory for full buffer, including space for trailing EOB's. */
    n = _yybytes_len + 2;
    buf = (char *) yyalloc(n ,yyscanner );
    if ( ! buf )
        YY_FATAL_ERROR( "out of dynamic memory in yy_scan_bytes()" );

    for ( i = 0; i < _yybytes_len; ++i )
        buf[i] = yybytes[i];

    buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

    b = yy_scan_buffer(buf,n ,yyscanner);
    if ( ! b )
        YY_FATAL_ERROR( "bad buffer in yy_scan_bytes()" );

    /* It's okay to grow etc. this buffer, and we should throw it
     * away when we're done.
     */
    b->yy_is_our_buffer = 1;

    return b;
}
/* %endif */

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

/* %if-c-only */
static void yy_fatal_error (yyconst char* msg , yyscan_t yyscanner)
{
        (void) fprintf( stderr, "%s\n", msg );
    exit( YY_EXIT_FAILURE );
}
/* %endif */
/* %if-c++-only */
/* %endif */

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
    do \
        { \
        /* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
        yytext[yyleng] = yyg->yy_hold_char; \
        yyg->yy_c_buf_p = yytext + yyless_macro_arg; \
        yyg->yy_hold_char = *yyg->yy_c_buf_p; \
        *yyg->yy_c_buf_p = '\0'; \
        yyleng = yyless_macro_arg; \
        } \
    while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/* %if-c-only */
/* %if-reentrant */

/** Get the user-defined data for this scanner.
 * @param yyscanner The scanner object.
 */
YY_EXTRA_TYPE yyget_extra  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyextra;
}

/* %endif */

/** Get the current line number.
 * @param yyscanner The scanner object.
 */
int yyget_lineno  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        if (! YY_CURRENT_BUFFER)
            return 0;

    return yylineno;
}

/** Get the current column number.
 * @param yyscanner The scanner object.
 */
int yyget_column  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        if (! YY_CURRENT_BUFFER)
            return 0;

    return yycolumn;
}

/** Get the input stream.
 * @param yyscanner The scanner object.
 */
FILE *yyget_in  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyin;
}

/** Get the output stream.
 * @param yyscanner The scanner object.
 */
FILE *yyget_out  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyout;
}

/** Get the length of the current token.
 * @param yyscanner The scanner object.
 */
int yyget_leng  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyleng;
}

/** Get the current token.
 * @param yyscanner The scanner object.
 */

char *yyget_text  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yytext;
}

/* %if-reentrant */

/** Set the user-defined data. This data is never touched by the scanner.
 * @param user_defined The data to be associated with this scanner.
 * @param yyscanner The scanner object.
 */
void yyset_extra (YY_EXTRA_TYPE  user_defined , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyextra = user_defined ;
}

/* %endif */

/** Set the current line number.
 * @param line_number
 * @param yyscanner The scanner object.
 */
void yyset_lineno (int  line_number , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        /* lineno is only valid if an input buffer exists. */
        if (! YY_CURRENT_BUFFER )
           yy_fatal_error( "yyset_lineno called with no buffer" , yyscanner);

    yylineno = line_number;
}

/** Set the current column.
 * @param line_number
 * @param yyscanner The scanner object.
 */
void yyset_column (int  column_no , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        /* column is only valid if an input buffer exists. */
        if (! YY_CURRENT_BUFFER )
           yy_fatal_error( "yyset_column called with no buffer" , yyscanner);

    yycolumn = column_no;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param in_str A readable stream.
 * @param yyscanner The scanner object.
 * @see yy_switch_to_buffer
 */
void yyset_in (FILE *  in_str , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyin = in_str ;
}

void yyset_out (FILE *  out_str , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyout = out_str ;
}

int yyget_debug  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yy_flex_debug;
}

void yyset_debug (int  bdebug , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yy_flex_debug = bdebug ;
}

/* %endif */

/* %if-reentrant */
/* Accessor methods for yylval and yylloc */

/* %if-bison-bridge */

YYSTYPE * yyget_lval  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yylval;
}

void yyset_lval (YYSTYPE *  yylval_param , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yylval = yylval_param;
}

/* %endif */

/* User-visible API */

/* yylex_init is special because it creates the scanner itself, so it is
 * the ONLY reentrant function that doesn't take the scanner as the last argument.
 * That's why we explicitly handle the declaration, instead of using our macros.
 */

int yylex_init(yyscan_t* ptr_yy_globals)

{
    if (ptr_yy_globals == NULL){
        errno = EINVAL;
        return 1;
    }

    *ptr_yy_globals = (yyscan_t) yyalloc ( sizeof( struct yyguts_t ), NULL );

    if (*ptr_yy_globals == NULL){
        errno = ENOMEM;
        return 1;
    }

    /* By setting to 0xAA, we expose bugs in yy_init_globals. Leave at 0x00 for releases. */
    memset(*ptr_yy_globals,0x00,sizeof(struct yyguts_t));

    return yy_init_globals ( *ptr_yy_globals );
}

/* %endif */

/* %if-c-only */
static int yy_init_globals (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from yylex_destroy(), so don't allocate here.
     */

    yyg->yy_buffer_stack = 0;
    yyg->yy_buffer_stack_top = 0;
    yyg->yy_buffer_stack_max = 0;
    yyg->yy_c_buf_p = (char *) 0;
    yyg->yy_init = 0;
    yyg->yy_start = 0;

    yyg->yy_start_stack_ptr = 0;
    yyg->yy_start_stack_depth = 0;
    yyg->yy_start_stack =  NULL;

/* Defined in main.c */
#ifdef YY_STDINIT
    yyin = stdin;
    yyout = stdout;
#else
    yyin = (FILE *) 0;
    yyout = (FILE *) 0;
#endif

    /* For future reference: Set errno on error, since we are called by
     * yylex_init()
     */
    return 0;
}
/* %endif */

/* %if-c-only SNIP! this currently causes conflicts with the c++ scanner */
/* yylex_destroy is for both reentrant and non-reentrant scanners. */
int yylex_destroy  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    /* Pop the buffer stack, destroying each element. */
    while(YY_CURRENT_BUFFER){
        yy_delete_buffer(YY_CURRENT_BUFFER ,yyscanner );
        YY_CURRENT_BUFFER_LVALUE = NULL;
        yypop_buffer_state(yyscanner);
    }

    /* Destroy the stack itself. */
    yyfree(yyg->yy_buffer_stack ,yyscanner);
    yyg->yy_buffer_stack = NULL;

    /* Destroy the start condition stack. */
        yyfree(yyg->yy_start_stack ,yyscanner );
        yyg->yy_start_stack = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * yylex() is called, initialization will occur. */
    yy_init_globals( yyscanner);

/* %if-reentrant */
    /* Destroy the main struct (reentrant only). */
    yyfree ( yyscanner , yyscanner );
    yyscanner = NULL;
/* %endif */
    return 0;
}
/* %endif */

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy (char* s1, yyconst char * s2, int n , yyscan_t yyscanner)
{
    register int i;
    for ( i = 0; i < n; ++i )
        s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * s , yyscan_t yyscanner)
{
    register int n;
    for ( n = 0; s[n]; ++n )
        ;

    return n;
}
#endif

void *yyalloc (yy_size_t  size , yyscan_t yyscanner)
{
    return (void *) malloc( size );
}

void *yyrealloc  (void * ptr, yy_size_t  size , yyscan_t yyscanner)
{
    /* The cast to (char *) in the following accommodates both
     * implementations that use char* generic pointers, and those
     * that use void* generic pointers.  It works with the latter
     * because both ANSI C and C++ allow castless assignment from
     * any pointer type to void*, and deal with argument conversions
     * as though doing an assignment.
     */
    return (void *) realloc( (char *) ptr, size );
}

void yyfree (void * ptr , yyscan_t yyscanner)
{
    free( (char *) ptr );   /* see yyrealloc() for (char *) cast */
}

/* %if-tables-serialization definitions */
/* %define-yytables   The name for this specific scanner's tables. */
#define YYTABLES_NAME "yytables"
/* %endif */

/* %ok-for-header */

#line 434 "pir.l"




static int
is_parrot_op(char const * const spelling) {

    /* only these are currently recognized as a Parrot instruction */
    if (strcmp(spelling, "print") == 0)
        return 1;
    if (strcmp(spelling, "null") == 0)
        return 1;
    if (strcmp(spelling, "new") == 0)
        return 1;
    if (strcmp(spelling, "newclass") == 0)
        return 1;
    if (strcmp(spelling, "end") == 0)
        return 1;
    if (strcmp(spelling, "setline") == 0)
        return 1;
    if (strcmp(spelling, "setfile") == 0)
        return 1;
    return 0;
}




/*
 * Local variables:
 *   c-file-style: "parrot"
 * End:
 * vim: expandtab shiftwidth=4:
 */

